import pytest
import json
import yaml

from . import ROOT_PATH


@pytest.mark.parametrize("basename", ["data", "schema"])
def test_data_sync(basename):
    """Verify that .json files are up-to-date with .yaml files."""
    yaml_path = ROOT_PATH / f"src/cdn_definitions/{basename}.yaml"
    json_path = ROOT_PATH / f"src/cdn_definitions/{basename}.json"

    with yaml_path.open(encoding="utf-8") as f:
        yaml_data = yaml.safe_load(f)

    with json_path.open(encoding="utf-8") as f:
        json_data = json.load(f)

    message = (
        f"{yaml_path} and {json_path} differ! Consider running `scripts/make-json'."
    )
    assert yaml_data == json_data, message



import pytest

from cdn_definitions import PathAlias, origin_aliases, rhui_aliases


def test_rhui_sanity():
    """rhui_aliases must exist with expected properties"""
    paths = rhui_aliases()

    # There should be some items
    assert paths

    # Paths should have src/dest the correct way around
    for path in paths:
        assert "/rhui" in path.src
        assert "/rhui" not in path.dest


def test_origin_sanity():
    """origin_aliases must exist with expected properties"""
    paths = origin_aliases()

    # There should be some items
    assert paths

    # Paths should relate to origin
    for path in paths:
        assert "/origin" in path.src
        assert "/origin" in path.dest


def test_enforced_absolute():
    """PathAlias raises if given relative paths"""
    with pytest.raises(ValueError) as exc_info:
        PathAlias(src="/src", dest="dest")
    assert "relative path" in str(exc_info.value)

    with pytest.raises(ValueError) as exc_info:
        PathAlias(src="src", dest="/dest")
    assert "relative path" in str(exc_info.value)


def test_enforced_nonequal():
    """PathAlias raises if given equal paths"""
    with pytest.raises(ValueError) as exc_info:
        PathAlias(src="/src", dest="/src")
    assert "/src cannot alias itself" in str(exc_info.value)



import os
import json

from frozendict import frozendict
from frozenlist2 import frozenlist
import pytest
import requests_mock

from cdn_definitions import load_data, load_schema


def test_can_load_custom_json_data(monkeypatch, tmpdir):
    json_file = tmpdir.join("myfile.json")
    json_file.write('{"hello": ["from", "json"]}')

    # If we set an env var pointing at the above JSON file, the library
    # should load it instead of the bundled data
    monkeypatch.setenv("CDN_DEFINITIONS_PATH", str(json_file))
    data = load_data()

    assert data == {"hello": ["from", "json"]}
    assert isinstance(data, frozendict)
    assert isinstance(data["hello"], frozenlist)


def test_can_load_custom_yaml_data(monkeypatch, tmpdir):
    yaml_file = tmpdir.join("myfile.yaml")
    yaml_file.write("hello:\n  - from\n  - yaml")

    # If we set an env var pointing at the above YAML file, the library
    # should load it instead of the bundled data
    monkeypatch.setenv("CDN_DEFINITIONS_PATH", str(yaml_file))
    data = load_data()

    assert data == {"hello": ["from", "yaml"]}
    assert isinstance(data, frozendict)
    assert isinstance(data["hello"], frozenlist)


@pytest.mark.parametrize(
    "file_name, file_contents, expected",
    [
        ("myfile.json", '{"french": "toast"}', {"french": "toast"}),
        ("myfile.yaml", "french: fries", {"french": "fries"}),
    ],
)
def test_can_load_local_data_from_source(tmpdir, file_name, file_contents, expected):
    json_file = tmpdir.join(file_name)
    json_file.write(file_contents)

    data = load_data(source=str(json_file))

    assert data == expected


def test_can_load_yaml_url_from_env_var(monkeypatch):
    with requests_mock.Mocker() as m:
        m.get("https://test.com/data.yaml", text='{"water": "melon"}')
        monkeypatch.setenv("CDN_DEFINITIONS_PATH", "https://test.com/data.yaml")
        data = load_data()

    assert data == {"water": "melon"}


def test_can_load_json_url_from_env_var(monkeypatch):
    with requests_mock.Mocker() as m:
        m.get("https://test.com/data.json", json={"green": "bean"})
        monkeypatch.setenv("CDN_DEFINITIONS_PATH", "https://test.com/data.json")
        data = load_data()

    assert data == {"green": "bean"}


def test_can_load_yaml_url_from_source_arg(monkeypatch):
    with requests_mock.Mocker() as m:
        m.get("http://test.com/data.yaml", text='{"grape": "fruit"}')
        data = load_data("http://test.com/data.yaml")

    assert data == {"grape": "fruit"}


def test_can_load_json_url_from_source_arg(monkeypatch):
    with requests_mock.Mocker() as m:
        m.get("http://test.com/data.json", json={"green": "pepper"})
        data = load_data("http://test.com/data.json")

    assert data == {"green": "pepper"}


def test_invalid_data_source():
    with pytest.raises(RuntimeError, match="Could not load data"):
        data = load_data(source="test")


def test_load_schema():
    with open(
        os.path.join(
            os.path.dirname(os.path.dirname(__file__)),
            "src",
            "cdn_definitions",
            "schema.json",
        )
    ) as f:
        local_schema = json.load(f)

    assert load_schema() == local_schema



import pytest
import yaml
import jsonschema

from jsonschema import Draft7Validator, ValidationError

from . import ROOT_PATH

DATA_PATH = ROOT_PATH / "src/cdn_definitions/data.yaml"
SCHEMA_PATH = ROOT_PATH / "src/cdn_definitions/schema.yaml"
HISTORICAL_DATA_DIR = ROOT_PATH / "tests/historical-data"


def validate(instance):
    with SCHEMA_PATH.open(encoding="utf-8") as f:
        schema = yaml.safe_load(f)
    jsonschema.validate(instance, schema, format_checker=Draft7Validator.FORMAT_CHECKER)


@pytest.fixture(name="data")
def fixture_data():
    with DATA_PATH.open(encoding="utf-8") as f:
        return yaml.safe_load(f)


def historical_data():
    for f in HISTORICAL_DATA_DIR.iterdir():
        with f.open(encoding="utf-8") as d:
            yield f.name, yaml.safe_load(d)


def test_data_matches_schema(data):
    """Verify that the content of data.yaml matches the declared schema."""

    validate(data)


@pytest.mark.parametrize("name, historical_data", historical_data())
def test_historical_data_matches_schema(name, historical_data):
    """Verify that all the historical data.yaml files that were compatible preceding
    the schema changes matches the currently declared schema i.e. current schema is
    backward compatible."""

    validate(historical_data)


def test_bogus_data_not_match_schema():
    """Verify that a completely bogus object does NOT match the schema.

    This is a sanity check to ensure that the schema is doing something."""

    with pytest.raises(ValidationError):
        validate({"bad": "data"})


def test_relative_path_not_match_schema(data):
    """Verify that data using a relative path does NOT match the schema."""

    data["rhui_alias"][0]["dest"] = "dest-path"
    with pytest.raises(ValidationError):
        validate(data)


def test_invalid_signing_key(data):
    """Verify that data using an invalid signing key does NOT match the schema."""

    data["signing_keys_mappings"]["platform"]["example_platform"]["ga_keys"] = "INVALID"
    with pytest.raises(ValidationError):
        validate(data)


@pytest.mark.parametrize(
    "platform_full_version, platform_version",
    [("7", "7"), ("5.2", "5.3"), ("7", "5.2"), ("5.2", "7")],
)
def test_major_or_minor_regex(data, platform_full_version, platform_version):
    """Validate the major_or_minor_version regex."""

    # cfme_version_mappings are a major_or_minor_version to major_or_minor_version mapping.
    data["cfme_version_mappings"][platform_full_version] = platform_version
    validate(data)


@pytest.mark.parametrize(
    "test_data", ["latest_rhel_6_dist", "latest_rhel6_ga", "current_rhel_dist"]
)
def test_current_latest_rhel_string(data, test_data):
    """Verify that data using invalid env_to_rhel_version_mappings strings do NOT
    match the schema."""

    data["env_to_releasever_mappings"]["prod"] = {test_data: "3.1"}
    data["env_to_releasever_mappings"]["stage"] = {test_data: "3.1"}
    data["env_to_releasever_mappings"]["qa"] = {test_data: "3.1"}
    with pytest.raises(ValidationError):
        validate(data)


def test_repo_override_not_match_schema(data):
    """Verify that a repo override without if_* condition does NOT match the schema."""

    del data["repo_overrides"]["stage"][0]["if_match_id"]
    with pytest.raises(ValidationError):
        validate(data)
